=== modified file 'NEWS'
--- NEWS	2005-10-14 10:37:07 +0000
+++ NEWS	2005-10-17 02:46:47 +0000
@@ -11,6 +11,9 @@
 
     * Improved zsh completion support in contrib/zsh, from Clint
       Adams.
+
+    * Builtin 'bzr annotate' command, by Martin Pool with improvements from 
+      Goffredo Baroncelli.
     
   INTERNALS:
 

=== modified file 'bzrlib/annotate.py'
--- bzrlib/annotate.py	2005-10-13 06:33:22 +0000
+++ bzrlib/annotate.py	2005-10-17 02:46:47 +0000
@@ -18,8 +18,6 @@
 
 # TODO: Choice of more or less verbose formats:
 # 
-# short: just show revno
-# long: revno, author, date
 # interposed: show more details between blocks of modified lines
 
 # TODO: Show which revision caused a line to merge into the parent
@@ -29,22 +27,41 @@
 import time
 
 import bzrlib.weave
-
-def annotate_file(branch, rev_id, file_id, to_file=None):
+from bzrlib.config import extract_email_address
+
+
+def annotate_file(branch, rev_id, file_id, verbose=False, full=False,
+        to_file=None):
     if to_file is None:
         to_file = sys.stdout
+
+    prevanno=''
+    for (revno_str, author, date_str, line_rev_id, text ) in \
+            _annotate_file(branch, rev_id, file_id ):
+
+        if verbose:
+            anno = '%5s %-12s %8s ' % (revno_str, author[:12], date_str)
+        else:
+            anno = "%5s %-7s " % ( revno_str, author[:7] )
+
+        if anno.lstrip() == "" and full: anno = prevanno
+        print >>to_file, '%s| %s' % (anno, text)
+        prevanno=anno
+
+def _annotate_file(branch, rev_id, file_id ):
+
     rh = branch.revision_history()
     w = branch.weave_store.get_weave(file_id, branch.get_transaction())
     last_origin = None
     for origin, text in w.annotate_iter(rev_id):
         text = text.rstrip('\r\n')
         if origin == last_origin:
-            anno = ''
+            (revno_str, author, date_str) = ('','','')
         else:
             last_origin = origin
             line_rev_id = w.idx_to_name(origin)
             if not branch.has_revision(line_rev_id):
-                anno = '???'
+                (revno_str, author, date_str) = ('?','?','?')
             else:
                 if line_rev_id in rh:
                     revno_str = str(rh.index(line_rev_id) + 1)
@@ -56,12 +73,12 @@
                                      time.gmtime(rev.timestamp + tz))
             # a lazy way to get something like the email address
             # TODO: Get real email address
-            author = line_rev_id
-            if '@' in author:
-                author = author[:author.index('@')]
-            author = author[:12]
-            anno = '%5s %-12s %8s' % (revno_str, author, date_str)
-        print '%-27.27s | %s' % (anno, text)
+            author = rev.committer
+            try:
+                author = extract_email_address(author)
+            except BzrError:
+                pass        # use the whole name
+        yield (revno_str, author, date_str, line_rev_id, text)
 
 
 if __name__ == '__main__':
@@ -74,4 +91,4 @@
     tree = b.revision_tree(b.last_revision())
     file_id = tree.inventory.path2id(rp)
     file_version = tree.inventory[file_id].revision
-    annotate_file(b, file_version, file_id, sys.stdout)
+    annotate_file(b, file_version, file_id, to_file = sys.stdout)

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-10-14 09:42:49 +0000
+++ bzrlib/builtins.py	2005-10-17 02:46:47 +0000
@@ -1455,16 +1455,23 @@
 class cmd_annotate(Command):
     """Show the origin of each line in a file.
 
-    This prints out the given file with an annotation on the 
-    left side indicating which revision, author and date introduced the 
-    change.
+    This prints out the given file with an annotation on the left side
+    indicating which revision, author and date introduced the change.
+
+    If the origin is the same for a run of consecutive lines, it is 
+    shown only at the top, unless the --all option is given.
     """
     # TODO: annotate directories; showing when each file was last changed
     # TODO: annotate a previous version of a file
+    # TODO: if the working copy is modified, show annotations on that 
+    #       with new uncommitted lines marked
     aliases = ['blame', 'praise']
     takes_args = ['filename']
+    takes_options = [Option('all', help='show annotations on all lines'),
+                     Option('long', help='show date in annotations'),
+                     ]
 
-    def run(self, filename):
+    def run(self, filename, all=False, long=False):
         from bzrlib.annotate import annotate_file
         b = Branch.open_containing(filename)
         b.lock_read()
@@ -1473,7 +1480,7 @@
             tree = b.revision_tree(b.last_revision())
             file_id = tree.inventory.path2id(rp)
             file_version = tree.inventory[file_id].revision
-            annotate_file(b, file_version, file_id, sys.stdout)
+            annotate_file(b, file_version, file_id, long, all, sys.stdout)
         finally:
             b.unlock()
 

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-10-14 09:37:21 +0000
+++ bzrlib/selftest/__init__.py	2005-10-17 02:46:47 +0000
@@ -33,7 +33,7 @@
 import bzrlib.osutils as osutils
 from bzrlib.selftest import TestUtil
 from bzrlib.selftest.TestUtil import TestLoader, TestSuite
-
+from bzrlib.selftest.treeshape import build_tree_contents
 
 MODULES_TO_TEST = []
 MODULES_TO_DOCTEST = []
@@ -422,6 +422,9 @@
                 print >>f, "contents of", name
                 f.close()
 
+    def build_tree_contents(self, shape):
+        bzrlib.selftest.build_tree_contents(shape)
+
     def failUnlessExists(self, path):
         """Fail unless path, which may be abs or relative, exists."""
         self.failUnless(osutils.lexists(path))

=== modified file 'bzrlib/selftest/testannotate.py'
--- bzrlib/selftest/testannotate.py	2005-10-13 06:33:22 +0000
+++ bzrlib/selftest/testannotate.py	2005-10-17 02:46:47 +0000
@@ -39,6 +39,21 @@
 
 
 class TestAnnotate(TestCaseInTempDir):
+    def setUp(self):
+        super(TestAnnotate, self).setUp()
+        b = Branch.initialize('.')
+        self.build_tree_contents([('hello.txt', 'my helicopter\n')])
+        b.add(['hello.txt'])
+        b.commit('add hello', 
+                 committer='test@user')
+
     def test_help_annotate(self):
         """Annotate command exists"""
         out, err = self.run_bzr_captured(['--no-plugins', 'annotate', '--help'])
+
+    def test_annotate_cmd(self):
+        out, err = self.run_bzr_captured(['annotate', 'hello.txt'])
+        self.assertEquals(err, '')
+        self.assertEqualDiff(out, '''\
+    1 test@us | my helicopter
+''')

